/* -*- Mode: C; c-basic-offset: 4 -*- */
/* GTK DBF Editor -- a simple dbf editor built with GTK+
 * Copyright (C) 2001 Steffen Macke
 *
 * callbacks.c: GTK+ callback handling
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <stdio.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "dbf.h"

/* open a file selection dialog */
void
on_open_file_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
       GtkWidget *vbox_fields;

       vbox_fields = lookup_widget(GTK_WIDGET(menuitem), "vbox_fields");

       create_dbf_file_selection(GTK_BOX(vbox_fields));
}

/* close main window, exit application, if possible */
void
on_exit1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    application_is_closing = TRUE;
    if(dbf_can_close() == TRUE) {
	gtk_exit(0);
    }
}

/* close main window, exit application, if possible */
gboolean
on_dbf_editor_window_delete_event      (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
    application_is_closing = TRUE;
    if(dbf_can_close() == TRUE) {
	gtk_exit(0);
    }
    else {
	gtk_main();
    }
    return FALSE;
}


/* save the dbf file */
void
on_save1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    dbf_save_file();
}

/* display help */
void
on_help1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}

/* close the dbf file, save if necessary */
void
on_close1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    GtkWidget *vbox_fields;

    if(dbf_can_close() == TRUE) {
	vbox_fields = lookup_widget(dbf_editor_window, "vbox_fields");
	dbf_remove_widgets(GTK_BOX(vbox_fields));
	gtk_widget_reset_shapes(dbf_editor_window);
	dbf_close_file();
	gtk_container_foreach(GTK_CONTAINER(lookup_widget(dbf_editor_window,
							  (const gchar *)
							  "hbox2")),
			      (GtkCallback)gtk_widget_set_sensitive, FALSE);
	window_display_status_message(dbf_editor_window, "");	
    }
}


/* create and display the about dialog */
void
on_about1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    GtkWidget *about1;
    GtkLabel  *label_version;

    about1 = create_about1();
    label_version = GTK_LABEL(lookup_widget(about1, "label_version"));
    gtk_label_set_text(label_version, VERSION);
    gtk_widget_show(about1);
}


/* close and destroy the about dialog */
void
on_about_ok_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *about1;

    about1 = lookup_widget(GTK_WIDGET(button), "about1");    
    gtk_widget_hide(about1);
    gtk_widget_destroy(about1);
}


void
on_go_first_record_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *dbf_editor_window;
    
    dbf_editor_window = lookup_widget(GTK_WIDGET(button), "dbf_editor_window");
    dbf_display_record(dbf_editor_window, 0);
}


void
on_go_previous_record_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *dbf_editor_window;
    
    dbf_editor_window = lookup_widget(GTK_WIDGET(button), "dbf_editor_window");    

    dbf_display_previous_record(dbf_editor_window);
}


void
on_go_next_record_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *dbf_editor_window;
    
    dbf_editor_window = lookup_widget(GTK_WIDGET(button), "dbf_editor_window");    

    dbf_display_next_record(dbf_editor_window);
}


void
on_go_last_record_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *dbf_editor_window;
    
    dbf_editor_window = lookup_widget(GTK_WIDGET(button), "dbf_editor_window");    
    dbf_display_last_record(dbf_editor_window);
}


void
on_add_record_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *dbf_editor_window;
    
    dbf_editor_window = lookup_widget(GTK_WIDGET(button), "dbf_editor_window");
    dbf_add_record(dbf_editor_window);
}


gboolean
on_entry1_focus_out_event              (GtkWidget       *widget,
                                        GdkEventFocus   *event,
                                        gpointer         user_data)
{

  return FALSE;
}

void
on_button_save_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *dialog_dbf_has_changed;
    GtkWidget *vbox_fields;

    dialog_dbf_has_changed = lookup_widget(GTK_WIDGET(button), 
					   "dialog_dbf_has_changed");
    gtk_widget_hide(dialog_dbf_has_changed);

    if(application_is_closing == TRUE) {
	dbf_close_file();
	gtk_exit(0);
    }
    else {
	dbf_save_file();

	vbox_fields = lookup_widget(dbf_editor_window, "vbox_fields");
	dbf_remove_widgets(GTK_BOX(vbox_fields));
	gtk_widget_reset_shapes(dbf_editor_window);
	dbf_close_file();
	gtk_container_foreach(GTK_CONTAINER(lookup_widget(dbf_editor_window,
							  (const gchar *)
							  "hbox2")),
			      (GtkCallback)gtk_widget_set_sensitive, FALSE);
	window_display_status_message(dbf_editor_window, "");
    }
}

void
on_button_no_save_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *dialog_dbf_has_changed;
    GtkWidget *vbox_fields;

    dialog_dbf_has_changed = lookup_widget(GTK_WIDGET(button), 
					   "dialog_dbf_has_changed");
    gtk_widget_hide(dialog_dbf_has_changed);

    if(application_is_closing == TRUE) {
	gtk_exit(0);
    }
    else {

	vbox_fields = lookup_widget(dbf_editor_window, "vbox_fields");
	dbf_remove_widgets(GTK_BOX(vbox_fields));
	gtk_widget_reset_shapes(dbf_editor_window);
	gtk_container_foreach(GTK_CONTAINER(lookup_widget(dbf_editor_window,
							  (const gchar *)
							  "hbox2")),
			      (GtkCallback)gtk_widget_set_sensitive, FALSE);
	window_display_status_message(dbf_editor_window, "");
	dbfh_file = NULL;
    }
}

/* cancel save and application closure */
void
on_button_cancel_save_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *dialog_dbf_has_changed;

    dialog_dbf_has_changed = lookup_widget(GTK_WIDGET(button), 
					   "dialog_dbf_has_changed");
    gtk_widget_hide(dialog_dbf_has_changed);
    
    application_is_closing = FALSE;
}



void
on_Table_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{   
    if(table_window == NULL) {
	table_window = create_table_window();
    }
    if(!GTK_WIDGET_VISIBLE(table_window)) {
	gtk_widget_show(table_window);
    }
    else {
	gtk_widget_hide(table_window);
    }
}

